from discord import TextChannel
from discord.ext import commands
from discord_slash import SlashContext, cog_ext
from discord_slash.utils.manage_commands import create_option

from jarvis.db.models import Autopurge, Purge
from jarvis.utils.permissions import admin_or_permissions


class PurgeCog(commands.Cog):
    def __init__(self, bot):
        self.bot = bot

    @cog_ext.cog_slash(
        name="purge",
        description="Purge messages from channel",
        options=[
            create_option(
                name="amount",
                description="Amount of messages to purge",
                required=False,
                option_type=4,
            )
        ],
    )
    @admin_or_permissions(manage_messages=True)
    async def _purge(self, ctx: SlashContext, amount: int = 10):
        if amount < 1:
            await ctx.send("Amount must be >= 1", hidden=True)
            return
        await ctx.defer()
        channel = ctx.channel
        messages = []
        async for message in channel.history(limit=amount + 1):
            messages.append(message)
        await channel.delete_messages(messages)
        _ = Purge(
            channel=ctx.channel.id,
            guild=ctx.guild.id,
            admin=ctx.author.id,
            count=amount,
        ).save()

    @cog_ext.cog_subcommand(
        base="autopurge",
        name="add",
        description="Automatically purge messages after x seconds",
        options=[
            create_option(
                name="channel",
                description="Channel to autopurge",
                option_type=7,
                required=True,
            ),
            create_option(
                name="delay",
                description="Seconds to keep message before purge, default 30",
                option_type=4,
                required=False,
            ),
        ],
    )
    @admin_or_permissions(manage_messages=True)
    async def _autopurge_add(
        self, ctx: SlashContext, channel: TextChannel, delay: int = 30
    ):
        if not isinstance(channel, TextChannel):
            await ctx.send("Channel must be a TextChannel", hidden=True)
            return
        if delay <= 0:
            await ctx.send("Delay must be > 0", hidden=True)
            return
        elif delay > 300:
            await ctx.send("Delay must be < 5 minutes", hidden=True)
            return
        autopurge = Autopurge.objects(
            guild=ctx.guild.id, channel=channel.id
        ).first()
        if autopurge:
            await ctx.send("Autopurge already exists.", hidden=True)
            return
        _ = Autopurge(
            guild=ctx.guild.id,
            channel=channel.id,
            admin=ctx.author.id,
            delay=delay,
        ).save()
        await ctx.send(
            f"Autopurge set up on {channel.mention}, "
            + f"delay is {delay} seconds"
        )

    @cog_ext.cog_subcommand(
        base="autopurge",
        name="remove",
        description="Remove an autopurge",
        options=[
            create_option(
                name="channel",
                description="Channel to remove from autopurge",
                option_type=7,
                required=True,
            ),
        ],
    )
    @admin_or_permissions(manage_messages=True)
    async def _autopurge_remove(self, ctx: SlashContext, channel: TextChannel):
        autopurge = Autopurge.objects(guild=ctx.guild.id, channel=channel.id)
        if not autopurge:
            await ctx.send("Autopurge does not exist.", hidden=True)
            return
        autopurge.delete()
        await ctx.send(f"Autopurge removed from {channel.mention}.")

    @cog_ext.cog_subcommand(
        base="autopurge",
        name="update",
        description="Update autopurge on a channel",
        options=[
            create_option(
                name="channel",
                description="Channel to update",
                option_type=7,
                required=True,
            ),
            create_option(
                name="delay",
                description="New time to save",
                option_type=4,
                required=True,
            ),
        ],
    )
    @admin_or_permissions(manage_messages=True)
    async def _autopurge_update(
        self, ctx: SlashContext, channel: TextChannel, delay: int
    ):
        autopurge = Autopurge.objects(guild=ctx.guild.id, channel=channel.id)
        if not autopurge:
            await ctx.send("Autopurge does not exist.", hidden=True)
            return
        autopurge.delay = delay
        autopurge.save()
        await ctx.send(
            f"Autopurge delay updated to {delay} seconds on {channel.mention}."
        )
